<?php
/*
 * Shoputils
 *
 * ПРИМЕЧАНИЕ К ЛИЦЕНЗИОННОМУ СОГЛАШЕНИЮ
 *
 * Этот файл связан лицензионным соглашением, которое можно найти в архиве,
 * вместе с этим файлом. Файл лицензии называется: LICENSE.1.5.x.RUS.txt
 * Так же лицензионное соглашение можно найти по адресу:
 * http://opencart.shoputils.ru/LICENSE.1.5.x.RUS.txt
 * 
 * =================================================================
 * OPENCART 1.5.x ПРИМЕЧАНИЕ ПО ИСПОЛЬЗОВАНИЮ
 * =================================================================
 *  Этот файл предназначен для Opencart 1.5.x. Shoputils не
 *  гарантирует правильную работу этого расширения на любой другой 
 *  версии Opencart, кроме Opencart 1.5.x. 
 *  Shoputils не поддерживает программное обеспечение для других 
 *  версий Opencart.
 * =================================================================
*/

class ControllerPaymentShoputilsIk extends Controller {
    private $order;
    private $log;
    private $shoputils = 0;
    private $version;
    private static $ACTION = 'https://sci.interkassa.com';
    private static $LOG_OFF = 0;
    private static $LOG_SHORT = 1;
    private static $LOG_FULL = 2;

    public function __construct($registry) {
        parent::__construct($registry);
        $this->load->language('payment/shoputils_ik');
    }

    protected function index() {
        $langdata = $this->config->get('shoputils_ik_langdata');
        $this->_setData(
            array(
                 'button_confirm',
                 'instruction'  => nl2br($langdata[$this->config->get('config_language_id')]['instruction']),
                 'continue'     => $this->url->link('checkout/success', '', 'SSL'),
                 'pay_status'   => ((!$this->config->get('shoputils_ik_laterpay_mode')) || ($this->config->get('shoputils_ik_order_later_status_id') == $this->config->get('shoputils_ik_order_confirm_status_id'))),
                 'action'       => self::$ACTION,
                 'parameters'   => $this->makeForm()
            )
        );

        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/shoputils_ik.tpl')) {
            $this->template = $this->config->get('config_template') . '/template/payment/shoputils_ik.tpl';
        } else {
            $this->template = 'default/template/payment/shoputils_ik.tpl';
        }

        $this->render();
    }

    public function laterpay() {
        if ($this->validateLaterpay()) {
        $this->_setData(
            array(
                 'action'     => self::$ACTION,
                 'parameters' => $this->makeForm($this->request->get['order_id'])
            )
        );

        if (file_exists(DIR_TEMPLATE . $this->config->get('config_template') . '/template/payment/shoputils_ik_laterpay.tpl')) {
            $this->template = $this->config->get('config_template') . '/template/payment/shoputils_ik_laterpay.tpl';
        } else {
            $this->template = 'default/template/payment/shoputils_ik_laterpay.tpl';
        }

        $this->response->setOutput($this->render());
        } else {
            $this->logWrite('Fail Validate Laterpay:', self::$LOG_FULL);
            $this->logWrite('  POST:' . var_export($this->request->post, true), self::$LOG_FULL);
            $this->logWrite('  GET:' . var_export($this->request->get, true), self::$LOG_FULL);
            $this->redirect($this->url->link('common/home', '', 'SSL'));
        }
    }

    public function status() {
        $this->logWrite('StatusURL: ', self::$LOG_FULL);
        $this->logWrite('  POST:' . var_export($this->request->post, true), self::$LOG_FULL);
        $this->logWrite('  GET:' . var_export($this->request->get, true), self::$LOG_FULL);

        if (!$this->validate(true)) {
            return;
        }

        if ($this->request->post['ik_inv_st']) { //ik_payment_state
            if ($this->order['order_status_id']){
                //Reverse $this->config->get('shoputils_ik_notify_customer_success')
                $notify = !$this->config->get('shoputils_ik_notify_customer_success');
                $this->model_checkout_order->update($this->order['order_id'],
                                                    $this->config->get('shoputils_ik_order_status_id'),
                                                    sprintf($this->language->get('text_comment'), $this->request->post['ik_pw_via'], $this->request->post['ik_am']),
                                                    $notify);

                if (!$notify) {
                    $langdata = $this->config->get('shoputils_ik_langdata');
                    $langdata = $langdata[(int)$this->config->get('config_language_id')];
                    $this->sendMail($langdata['mail_customer_success_subject'],
                                    $langdata['mail_customer_success_content'],
                                    $this->config->get('shoputils_ik_order_status_id'),
                                    'customer',
                                    'Mail to Customer Sent Successfully: Payment Success');
                }

                if ($this->config->get('shoputils_ik_notify_admin_success')) {
                    $this->sendMail($this->config->get('shoputils_ik_mail_admin_success_subject'),
                                    $this->config->get('shoputils_ik_mail_admin_success_content'),
                                    $this->config->get('shoputils_ik_order_status_id'),
                                    'admin',
                                    'Mail to Admin Sent Successfully: Payment Success');
                }
            }
        }

        $this->sendOk();
    }

    public function success() {
        $this->logWrite('SuccessURL', self::$LOG_FULL);
        $this->logWrite('  POST:' . var_export($this->request->post, true), self::$LOG_FULL);
        $this->logWrite('  GET:' . var_export($this->request->get, true), self::$LOG_FULL);

        if ($this->validate(false)) {
            $this->redirect($this->url->link('checkout/success', '', 'SSL'));
        } else {
            $this->redirect($this->url->link('common/home', '', 'SSL'));
            //$this->redirect($this->url->link('checkout/checkout', '', 'SSL'));
        }
    }

    public function fail() {
        $this->logWrite('FailURL', self::$LOG_FULL);
        $this->logWrite('  POST:' . var_export($this->request->post, true), self::$LOG_FULL);
        $this->logWrite('  GET:' . var_export($this->request->get, true), self::$LOG_FULL);
        if ($this->validate(false)) {
            if ($this->order['order_status_id']) {
                //Reverse $this->config->get('shoputils_ik_notify_customer_fail')
                $notify = !$this->config->get('shoputils_ik_notify_customer_fail');

                $this->model_checkout_order->update($this->order['order_id'],
                                                    $this->config->get('shoputils_ik_order_fail_status_id'),
                                                    sprintf($this->language->get('text_comment'), $this->request->post['ik_pw_via'], $this->request->post['ik_am']),
                                                    $notify);

              if (!$notify) {
                  $langdata = $this->config->get('shoputils_ik_langdata');
                  $langdata = $langdata[(int)$this->config->get('config_language_id')];
                  $this->sendMail($langdata['mail_customer_fail_subject'],
                                  $langdata['mail_customer_fail_content'],
                                  $this->config->get('shoputils_ik_order_fail_status_id'),
                                  'customer',
                                  'Mail to Customer Sent Successfully: Payment Fail');
              }

              if ($this->config->get('shoputils_ik_notify_admin_fail')) {
                  $this->sendMail($this->config->get('shoputils_ik_mail_admin_fail_subject'),
                                       $this->config->get('shoputils_ik_mail_admin_fail_content'),
                                       $this->config->get('shoputils_ik_order_fail_status_id'),
                                       'admin',
                                       'Mail to Admin Sent Successfully: Payment Fail');
              }
            }
        }

        $this->redirect($this->url->link('common/home', '', 'SSL'));
        //$this->redirect($this->url->link('checkout/checkout', '', 'SSL'));
    }

    public function confirm() {
        if (!empty($this->session->data['order_id'])) {
            $this->load->model('checkout/order');
            $this->model_checkout_order->confirm($this->session->data['order_id'], $this->config->get('shoputils_ik_order_confirm_status_id'));
        }
    }

    public function ik() {
        $oc = VERSION;
        if (file_exists(DIR_SYSTEM . 'config/svn/svn.ver')) {
          $oc .= '.r' . trim(file_get_contents(DIR_SYSTEM . 'config/svn/svn.ver'));
        }
        echo '<meta http-equiv="Content-Type" content="text/html; charset=utf-8"/>';
        echo sprintf('%s - %s v%s<br />', $oc, $this->shoputils, $this->config->get('shoputils_ik_version'));
        //info
        if ((isset($this->request->get['info'])) && (md5($this->request->get['info']) == 'caf9b6b99962bf5c2264824231d7a40c')) {
          $this->load->model('setting/setting');
          $ik = $this->model_setting_setting->getSetting('shoputils_ik');
          $disallowed = array(
                'shoputils_ik_shop_id',
                'shoputils_ik_sign_hash',
                'shoputils_ik_sign_test_key'
          );
          foreach ($ik as $key => $value) {
            if (!in_array($key, $disallowed)) {
              if (is_array($value)) {
              echo sprintf('%s:<br />', $key);
                foreach ($value as $k => $val) {
                  if (is_array($val)) {
                    echo sprintf('............%s:<br />', $k);
                    foreach ($val as $k2 => $v) {
                      if (is_array($v)) {
                        echo sprintf('........................%s:<br />', $k2);
                        foreach ($v as $k3 => $v2) {
                          echo sprintf('........................%s: %s<br /><br />', $k3, $v2);
                        }
                      } else{
                        echo sprintf('....................................%s: %s<br /><br />', $k2, $v);
                      }
                    }                
                  } else {
                    echo sprintf('............%s: %s<br /><br />', $k, $val);
                  }
                }
              } else {
                echo sprintf('%s: %s<br /><br />', $key, $value);
              }
            }
          }
        }
    }

    protected function makeForm($order_id = false) {
        $this->load->model('checkout/order');
        if (!$order_id ) {
          if (isset($this->session->data['order_id'])) {
            $order_id  = $this->session->data['order_id'];
          } else {
            $this->logWrite('Error: Unsupported Checkout Extension', self::$LOG_SHORT);
            die($this->language->get('error_fail_checkout_extension'));
          }
        }
        $order_info = $this->model_checkout_order->getOrder($order_id);

        $ikCurrencyCode = $this->config->get('shoputils_ik_currency');
        if (!$this->currency->has($ikCurrencyCode)) {
            die(sprintf('Currency code (for code: %s) not found', $ikCurrencyCode));
        }
        
        $ik_payment_amount = number_format($this->currency->convert($order_info['total'], $this->config->get('config_currency'), $ikCurrencyCode), 2, '.', '');
        //$ik_payment_amount = number_format($this->currency->format($order_info['total'], $this->config->get('shoputils_ik_currency'), $this->currency->getValue($this->config->get('shoputils_ik_currency')), false), 2, '.', '');
        $ik_shop_id       = $this->config->get('shoputils_ik_shop_id');
        $ik_payment_desc  = sprintf($this->language->get('text_ik_payment_desc'), $order_info['order_id']);
        $ik_lifetime      = $this->config->get('shoputils_ik_lifetime');

        $params = array(
            'ik_am'     => $ik_payment_amount,
            'ik_co_id'  => $ik_shop_id,
            'ik_cur'    => $ikCurrencyCode,
            'ik_desc'   => $ik_payment_desc,
            'ik_ltm'    => $ik_lifetime,
            'ik_pm_no'  => $order_info['order_id']
        );
        $params['ik_sign'] = base64_encode(md5(implode(':', array_values($params)) . ':' . $this->config->get('shoputils_ik_sign_hash'), true));

        $this->logWrite('Make payment form: ', self::$LOG_FULL);
        $this->logWrite('  DATA: ' . var_export($params, true), self::$LOG_FULL);

        return $params;
    }

    protected function validateLaterpay() {
        if ((!isset($this->request->get['order_id'])) || (!isset($this->request->get['order_tt'])) || (!isset($this->request->get['order_fl']))) {
          return false;
        } else {
          $this->load->model('checkout/order');
          $order_info = $this->model_checkout_order->getOrder($this->request->get['order_id']);
          if ((!$order_info) || ($this->request->get['order_id'] != $order_info['order_id']) || ($this->request->get['order_tt'] != $order_info['total']) || ($this->request->get['order_fl']) != $order_info['firstname'] . $order_info['lastname']) {
            return false;
          }
        }
        return true;
    }

    protected function validate($check_sign_hash = true) {
        $this->load->model('checkout/order');

        if ($this->request->server['REQUEST_METHOD'] != 'POST') {
            $this->sendForbidden($this->language->get('text_error_post'));
            return false;
        }

        if ($check_sign_hash && isset($sign_ik)) {
            $ik_sign_hash_array = $this->request->post;
            unset($ik_sign_hash_array['ik_sign']);
            ksort($ik_sign_hash_array, SORT_STRING);
            array_push($ik_sign_hash_array, $this->config->get('shoputils_ik_test_mode') ? $this->config->get('shoputils_ik_sign_test_key') : $this->config->get('shoputils_ik_sign_hash')); //$this->config->get('shoputils_ik_sign_hash');
            $ik_sign_hash_string = implode(':', $ik_sign_hash_array);
            $ik_sign_hash = base64_encode(md5($ik_sign_hash_string, true));

            if ($this->request->post['ik_sign'] != $ik_sign_hash) {  //ik_sign_hash
                $this->sendForbidden($this->language->get('text_error_ik_sign_hash'));
                $this->logWrite($ik_sign_hash . '=md5(' . $ik_sign_hash_string . ')', self::$LOG_SHORT);
                return false;
            }
        }

        $this->order = $this->model_checkout_order->getOrder($this->request->post['ik_pm_no']); //ik_payment_id

        if (!$this->order) {
            $this->sendForbidden(sprintf($this->language->get('text_error_order_not_found'), $this->request->post['ik_pm_no']));  //ik_payment_id
            return false;
        }

        return true;
    }

    protected function sendForbidden($error) {
        $this->logWrite('ERROR: ' . $error, self::$LOG_SHORT);
        header('HTTP/1.1 403 Forbidden');
        echo "<html>
                <head>
                   <title>403 Forbidden</title>
                </head>
                <body>
                    <p>$error.</p>
                </body>
        </html>";
    }

    protected function sendOk() {
        $this->logWrite('OK: ' . http_build_query($this->request->post, '', ','), self::$LOG_SHORT);
        header('HTTP/1.1 200 OK');
        echo "<html><head><title>200 OK</title></head></html>";
    }

    //type = 'admin' - mail send to admin; type = 'customer' - mail send to customer
    protected function sendMail($subject, $content, $order_status_id, $type = 'admin', $log_result = '') {
				$this->load->model('payment/shoputils_ik');

				$order_info = $this->model_payment_shoputils_ik->getOrder($this->order['order_id']);

				$input = array(
            '{order_id}',
				    '{store_name}',
						'{logo}',
						'{products}',
						'{total}',
						'{customer_firstname}',
						'{customer_lastname}',
						'{customer_group}',
						'{customer_email}',
						'{customer_telephone}',
						'{order_status}',
						'{comment}',
						'{ip}',
						'{date_added}',
						'{date_modified}'
			  );

				$output = array(
            'order_id'            => $order_info['order_id'],
				    'store_name'          => $this->config->get('config_name'),
				    'logo'                => '<a href="' . HTTP_SERVER . '"><img src="' . HTTP_SERVER . 'image/' . $this->config->get('config_logo') . '" / ></a>',
						'products'            => $this->model_payment_shoputils_ik->getProducts($order_info['order_id']),
						'total'               => $this->currency->format($order_info['total'], $order_info['currency_code'], $order_info['currency_value']),
						'customer_firstname'  => $order_info['payment_firstname'],
						'customer_lastname'   => $order_info['payment_lastname'],
						'customer_group'      => $this->model_payment_shoputils_ik->getCustomerGroup($order_info['customer_group_id']),
						'customer_email'      => $order_info['email'],
						'customer_telephone'  => $order_info['telephone'],
						'order_status'        => $this->model_payment_shoputils_ik->getOrderStatusById($order_status_id, $order_info['language_id']),
						'comment'             => $order_info['comment'],
						'ip'                  => $order_info['ip'],
						'date_added'          => $order_info['date_added'],
						'date_modified'       => $order_info['date_modified']
			  );

		    $subject = html_entity_decode(trim(str_replace($input, $output, $subject)), ENT_QUOTES, 'UTF-8');
		    $content = html_entity_decode(str_replace($input, $output, $content), ENT_QUOTES, 'UTF-8');
		    $to = $type == 'admin' ? $this->config->get('config_email') : $order_info['email'];
		    $this->_sendMail($subject, $content, $type, $to);
		    $this->logWrite($log_result, self::$LOG_FULL);
    }

    protected function _sendMail($subject, $content, $type, $to) {
				$message  = '<html dir="ltr" lang="en">' . "\n";
				$message .= '  <head>' . "\n";
				$message .= '    <title>' . $subject . '</title>' . "\n";
				$message .= '    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">' . "\n";
				$message .= '  </head>' . "\n";
				$message .= '  <body>' . $content . '</body>' . "\n";
				$message .= '</html>' . "\n";

				$mail = new Mail();
				$mail->protocol = $this->config->get('config_mail_protocol');
				$mail->parameter = $this->config->get('config_mail_parameter');
				$mail->hostname = $this->config->get('config_smtp_host');
				$mail->username = $this->config->get('config_smtp_username');
				$mail->password = $this->config->get('config_smtp_password');
				$mail->port = $this->config->get('config_smtp_port');
				$mail->timeout = $this->config->get('config_smtp_timeout');
				$mail->setTo($to);
        $mail->setFrom($this->config->get('config_email'));
				$mail->setSender($this->config->get('config_name'));
				$mail->setSubject($subject);
				$mail->setHtml($message);
				$mail->send();

				if ($type == 'admin') {
          $emails = explode(',', $this->config->get('config_alert_emails'));
          foreach ($emails as $email) {
            if ($email && preg_match('/^[^\@]+@.*\.[a-z]{2,6}$/i', $email)) {
              $mail->setTo($email);
              $mail->send();
            }
          }
				}
    }

    protected function _setData($values) {
        foreach ($values as $key => $value) {
            if (is_int($key)) {
                $this->data[$value] = $this->language->get($value);
            } else {
                $this->data[$key] = $value;
            }
        }
    }

    protected function logWrite($message, $type) {
        switch ($this->config->get('shoputils_ik_log')) {
            case self::$LOG_OFF:
                return;
            case self::$LOG_SHORT:
                if ($type == self::$LOG_FULL) {
                    return;
                }
        }

        if (!$this->log) {
            $this->log = new Log($this->config->get('shoputils_ik_log_filename'));
        }
        $this->log->Write($message);
    }
}
?>